#include <arpa/inet.h>
#include <stdio.h>
#include <string.h>

#include <zephyr/kernel.h>
#include <zephyr/drivers/clock_control.h>
#include <zephyr/drivers/clock_control/nrf_clock_control.h>

#include <nrf_modem.h>
#include <modem/nrf_modem_lib.h>
#include <modem/nrf_modem_lib_trace.h>
#include <modem/trace_backend.h>


/* To strictly comply with UART timing, enable external XTAL oscillator */
static void enable_xtal(void)
{
    struct onoff_manager *clk_mgr;
    static struct onoff_client cli = {};

    clk_mgr = z_nrf_clock_control_get_onoff(CLOCK_CONTROL_NRF_SUBSYS_HF);
    sys_notify_init_spinwait(&cli.notify);
    (void)onoff_request(clk_mgr, &cli);
}

static trace_backend_processed_cb trace_processed_callback;

int trace_backend_init(trace_backend_processed_cb processed)
{
    /* initialize transport backend here */
    printk("%s\n", __func__);
    trace_processed_callback = processed;
    return 0;
}

int trace_backend_deinit(void)
{
    /* optional deinitialization code here */
    printk("%s\n", __func__);
    return 0;
}

static void dump_buffer(const void *buf, uint32_t length)
{
    for (uint32_t i = 0; i < length; ) {
        if (!((int)buf & 0x3) && length - i >= 8) {
            printk("%08x%08x", htonl(*((const uint32_t *)(const void *)((const uint8_t *)buf + i))), htonl(*((const uint32_t *)(const void *)((const uint8_t *)buf + i + 4))));
            i += 8;
        } else {
            printk("%02x", *((uint8_t *)buf + i));
            i++;
        }
    }
}

int trace_backend_write(const void *data, size_t len)
{
    printk("%s len=%d ", __func__, len);
    dump_buffer(data, len);
    printk("\n");
    int r = len;
    int err = trace_processed_callback(r);
    if (err) {
        printk("%s err=%d\n", __func__, err);
        return err;
    }
    return r;
}

struct nrf_modem_lib_trace_backend trace_backend = {
   .init = trace_backend_init,
   .deinit = trace_backend_deinit,
   .write = trace_backend_write,
   .data_size = NULL,
   .read = NULL,
   .clear = NULL,
};

void nrf_modem_fault_handler(struct nrf_modem_fault_info *fault_info)
{
    printk(":<Mfh t=%u reason=%u pc=%08x\n", k_uptime_get_32(), fault_info->reason, fault_info->program_counter);
}


/**
 * Application entry point: initialization and main loop.
 */
void main(void)
{
    enable_xtal();

    printk("Hello, world\n");
    uint32_t i;
    for (i = 0; i < 10; i++) {
        printk("%d\n", i);
        k_sleep(K_MSEC(1000));
    }

    int err = nrf_modem_lib_init();
    printf("main: err=%d\n", err);
    while (1) {
        printk("zzz\n");
        k_sleep(K_MSEC(1000));
    }
}


/* End */
